@@ -1,5 +1,14 @@
-from django.conf.urls.defaults import *
-from views import empty_view, absolute_kwargs_view
+from __future__ import absolute_import
+
+from django.conf.urls import patterns, url, include
+
+from .views import empty_view, absolute_kwargs_view
+
+
+other_patterns = patterns('',
+    url(r'non_path_include/$', empty_view, name='non_path_include'),
+    url(r'nested_path/$', 'regressiontests.urlpatterns_reverse.views.nested_view'),
+)
 
 urlpatterns = patterns('',
     url(r'^places/(\d+)/$', empty_view, name='places'),
@@ -14,8 +23,9 @@ urlpatterns = patterns('',
     url(r'^people/(?P<name>\w+)/$', empty_view, name="people"),
     url(r'^people/(?:name/)', empty_view, name="people2"),
     url(r'^people/(?:name/(\w+)/)?', empty_view, name="people2a"),
+    url(r'^people/(?P<name>\w+)-(?P=name)/$', empty_view, name="people_backref"),
     url(r'^optional/(?P<name>.*)/(?:.+/)?', empty_view, name="optional"),
-    url(r'^hardcoded/$', 'hardcoded/', empty_view, name="hardcoded"),
+    url(r'^hardcoded/$', empty_view, name="hardcoded"),
     url(r'^hardcoded/doc\.pdf$', empty_view, name="hardcoded2"),
     url(r'^people/(?P<state>\w\w)/(?P<name>\w+)/$', empty_view, name="people3"),
     url(r'^people/(?P<state>\w\w)/(?P<name>\d)/$', empty_view, name="people4"),
@@ -52,4 +62,9 @@ urlpatterns = patterns('',
     url(r'absolute_arg_view/(?P<arg1>\d+)/$', absolute_kwargs_view),
     url(r'absolute_arg_view/$', absolute_kwargs_view),
 
+    # Tests for #13154. Mixed syntax to test both ways of defining URLs.
+    url(r'defaults_view1/(?P<arg1>\d+)/', 'defaults_view', {'arg2': 1}, name='defaults'),
+    (r'defaults_view2/(?P<arg1>\d+)/', 'defaults_view', {'arg2': 2}, 'defaults'),
+
+    url('^includes/', include(other_patterns)),
 )
